@@ -184,7 +184,7 @@ class RelaxationAnalyzer(object):
             there is no reason to duplicate the information or computation.
         """
         data = collections.defaultdict(dict)
-        for inds in itertools.combinations(xrange(len(self.initial)), 2):
+        for inds in itertools.combinations(list(range(len(self.initial))), 2):
             (i, j) = sorted(inds)
             initial_dist = self.initial[i].distance(self.initial[j])
             if initial_dist < max_radius:
@@ -318,9 +318,13 @@ def solid_angle(center, coords):
     r.append(r[0])
     n = [np.cross(r[i + 1], r[i]) for i in range(len(r) - 1)]
     n.append(np.cross(r[1], r[0]))
-    phi = sum([math.acos(-np.dot(n[i], n[i + 1])
-                         / (np.linalg.norm(n[i]) * np.linalg.norm(n[i + 1])))
-               for i in range(len(n) - 1)])
+    vals = []
+    for i in range(len(n) - 1):
+        v = -np.dot(n[i], n[i + 1])\
+            / (np.linalg.norm(n[i]) * np.linalg.norm(n[i + 1]))
+        v = max(min(v, 1), -1)
+        vals.append(math.acos(v))
+    phi = sum(vals)
     return phi + (3 - len(r)) * math.pi
 
 
