@@ -15,18 +15,23 @@ __date__ = "Nov 10, 2012"
 import re
 import collections
 import string
+
+import six
+from six.moves import filter
+from six.moves import map
+from six.moves import zip
+
 from fractions import gcd
 from functools import total_ordering
 from itertools import chain
-from monty.dev import deprecated
 from pymatgen.core.periodic_table import get_el_sp, Element
 from pymatgen.util.string_utils import formula_double_format
-from pymatgen.serializers.json_coders import MSONable
+from pymatgen.serializers.json_coders import PMGSONable
 from pymatgen.core.units import unitized
 
 
 @total_ordering
-class Composition(collections.Mapping, collections.Hashable, MSONable):
+class Composition(collections.Mapping, collections.Hashable, PMGSONable):
     """
     Represents a Composition, which is essentially a {element:amount} mapping
     type. Composition is written to be immutable and hashable,
@@ -105,11 +110,11 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
                 ambiguity.
         """
         self.allow_negative = kwargs.pop('allow_negative', False)
-        if len(args) == 1 and isinstance(args[0], basestring):
+        if len(args) == 1 and isinstance(args[0], six.string_types):
             elmap = self._parse_formula(args[0])
         else:
             elmap = dict(*args, **kwargs)
-        for k, v in elmap.items():
+        for k, v in list(elmap.items()):
             if v < -Composition.amount_tolerance and not self.allow_negative:
                 raise CompositionError("Amounts in Composition cannot be "
                                        "negative!")
@@ -152,7 +157,7 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
         """
         new_el_map = collections.defaultdict(float)
         new_el_map.update(self)
-        for k, v in other.iteritems():
+        for k, v in other.items():
             new_el_map[get_el_sp(k)] += v
         return Composition(new_el_map, allow_negative=self.allow_negative)
 
@@ -167,7 +172,7 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
         """
         new_el_map = collections.defaultdict(float)
         new_el_map.update(self)
-        for k, v in other.iteritems():
+        for k, v in other.items():
             new_el_map[get_el_sp(k)] -= v
         return Composition(new_el_map, allow_negative=self.allow_negative)
 
@@ -254,7 +259,7 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
         e.g., Li4 Fe4 P4 O16.
         """
         sym_amt = self.get_el_amt_dict()
-        syms = sorted(sym_amt.keys(), key=lambda s: get_el_sp(s).X)
+        syms = sorted(sym_amt.keys(), key=lambda sym: get_el_sp(sym).X)
         formula = [s + formula_double_format(sym_amt[s], False) for s in syms]
         return " ".join(formula)
 
@@ -329,17 +334,6 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
 
         return formula, factor
 
-    @deprecated(fractional_composition) #as of 8/20/14
-    def get_fractional_composition(self):
-        """
-        Returns the normalized composition which the number of species sum to
-        1.
-
-        Returns:
-            Normalized composition which the number of species sum to 1.
-        """
-        return self.fractional_composition
-
     @property
     def reduced_formula(self):
         """
@@ -353,12 +347,12 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
         """
         Returns view of elements in Composition.
         """
-        return self._elmap.keys()
+        return list(self._elmap.keys())
 
     def __str__(self):
         return " ".join([
             "{}{}".format(k, formula_double_format(v, ignore_ones=False))
-            for k, v in self.to_dict.items()])
+            for k, v in self.as_dict().items()])
 
     @property
     def num_atoms(self):
@@ -434,16 +428,6 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
             return self._parse_formula(expanded_formula)
         return get_sym_dict(formula, 1)
 
-    @staticmethod
-    @deprecated(__init__)
-    def from_formula(formula):
-        """
-        .. deprecated:: 1.6.1
-
-        Use Composition(formula) instead.
-        """
-        return Composition(formula)
-
     @property
     def anonymized_formula(self):
         """
@@ -473,10 +457,10 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
     @classmethod
     def from_dict(cls, d):
         """
-        Creates a composition from a dict generated by to_dict. Strictly not
+        Creates a composition from a dict generated by as_dict(). Strictly not
         necessary given that the standard constructor already takes in such an
         input, but this method preserves the standard pymatgen API of having
-        from_dict methods to reconstitute objects generated by to_dict. Allows
+        from_dict methods to reconstitute objects generated by as_dict(). Allows
         for easier introspection.
 
         Args:
@@ -495,8 +479,7 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
             d[e.symbol] += a
         return d
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         """
         Returns:
             dict with species symbol and (unreduced) amount e.g.,
@@ -515,7 +498,7 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
             {"Fe": 2.0, "O":3.0}
         """
         c = Composition(self.reduced_formula)
-        return c.to_dict
+        return c.as_dict()
 
     @property
     def to_data_dict(self):
@@ -526,10 +509,10 @@ class Composition(collections.Mapping, collections.Hashable, MSONable):
             reduced_cell_formula, elements and nelements.
         """
         return {"reduced_cell_composition": self.to_reduced_dict,
-                "unit_cell_composition": self.to_dict,
+                "unit_cell_composition": self.as_dict(),
                 "reduced_cell_formula": self.reduced_formula,
-                "elements": self.to_dict.keys(),
-                "nelements": len(self.to_dict.keys())}
+                "elements": self.as_dict().keys(),
+                "nelements": len(self.as_dict().keys())}
 
     @staticmethod
     def ranked_compositions_from_indeterminate_formula(fuzzy_formula,
@@ -741,13 +724,14 @@ def reduce_formula(sym_amt):
     syms = sorted(sym_amt.keys(),
                   key=lambda s: get_el_sp(s).X)
 
-    syms = filter(lambda s: abs(sym_amt[s]) > Composition.amount_tolerance, syms)
+    syms = list(filter(lambda s: abs(sym_amt[s]) >
+                                 Composition.amount_tolerance, syms))
     num_el = len(syms)
     contains_polyanion = (num_el >= 3 and
                           get_el_sp(syms[num_el - 1]).X
                           - get_el_sp(syms[num_el - 2]).X < 1.65)
 
-    factor = abs(reduce(gcd, sym_amt.values()))
+    factor = abs(six.moves.reduce(gcd, sym_amt.values()))
     reduced_form = []
     n = num_el - 2 if contains_polyanion else num_el
     for i in range(0, n):
