@@ -25,8 +25,15 @@ class Enum(set):
 
 class AttrDict(dict):
     """
-    Allows to access dict keys as obj.foo in addition to the traditional way
-    obj['foo']"
+    Allows to access dict keys as obj.foo in addition
+    to the traditional way obj['foo']"
+
+    >>> d = AttrDict(foo=1, bar=2)
+    >>> d["foo"] == d.foo
+    True
+    >>> d.bar = "hello"
+    >>> d.bar
+    'hello'
     """
     def __init__(self, *args, **kwargs):
         super(AttrDict, self).__init__(*args, **kwargs)
@@ -39,10 +46,13 @@ class AttrDict(dict):
 
 class NullFile(object):
     """A file object that is associated to /dev/null."""
-    def __init__(self):
+    def __new__(cls):
         import os
         return open(os.devnull, 'w')
 
+    def __init__(self):
+        """no-op"""
+
 
 class NullStream(object):
     """A fake stream with a no-op write.."""
