@@ -5,17 +5,21 @@ import os
 import time
 import collections
 import yaml
-from six.moves import cStringIO
-from monty.os.path import which
-from pymatgen.io.abinitio import myaml
 
+from six.moves import cStringIO
 from datetime import timedelta
+from monty.os.path import which
+from monty.dev import deprecated
 from pymatgen.core.design_patterns import AttrDict
 from pymatgen.util.string_utils import is_string
-from pymatgen.io.abinitio.utils import File
+
+try:
+    import apscheduler
+    has_sched_v3 = apscheduler.version >= "3.0.0"
+except ImportError:
+    pass
 
 import logging
-
 logger = logging.getLogger(__name__)
 
 __all__ = [
@@ -210,15 +214,20 @@ class PyLauncher(object):
     """
     Error = PyLauncherError
 
-    def __init__(self, flow):
+    def __init__(self, flow, **kwargs):
         """
         Initialize the object
 
         Args:
             flow:
                 `AbinitFlow` object
+            kwargs:
+                max_njobs_inqueue:
+                    The launcher will stop submitting jobs when the
+                    number of jobs in the queue is >= Max number of jobs
         """
         self.flow = flow
+        self.max_njobs_inqueue = kwargs.get("max_njobs_inqueue", 500)
 
     def single_shot(self):
         """
@@ -254,64 +263,73 @@ class PyLauncher(object):
 
         return num_launched
 
-    def rapidfire(self, max_nlaunch=-1, max_loops=1, sleep_time=None): # nlaunches=0,
+    def rapidfire(self, max_nlaunch=-1, max_loops=1, sleep_time=5): 
         """
         Keeps submitting `Tasks` until we are out of jobs or no job is ready to run.
 
         Args:
             max_nlaunch:
                 Maximum number of launches. default: no limit.
-
-        nlaunches:
-            0 means 'until completion', -1 or "infinite" means to loop forever
-        max_loops:
-            maximum number of loops
-        sleep_time:
-            secs to sleep between rapidfire loop iterations
+            max_loops:
+                Maximum number of loops
+            sleep_time:
+                secs to sleep between rapidfire loop iterations
 
         Returns:
             The number of tasks launched.
         """
-        sleep_time = sleep_time if sleep_time else 5
-        #nlaunches = -1 if nlaunches == 'infinite' else int(nlaunches)
-        num_loops, num_launched, launched = 0, 0, []
-        got_empty_list = 0
+        num_launched, launched = 0, []
+
+        for count in range(max_loops):
+            if count > 0:
+                time.sleep(sleep_time)
 
-        while num_loops != max_loops:
             tasks = self.fetch_tasks_to_run()
 
             # I don't know why but we receive duplicated tasks.
-            for task in tasks:
-                if task in launched:
-                    err_msg = "task %s already in launched list:\n%s" % (task, launched)
-                    logger.critical(err_msg)
+            if any(task in launched for task in tasks):
+                logger.critical("task %s already in launched list:\n%s" % (task, launched))
 
             # Preventive test.
             tasks = [t for t in tasks if t not in launched]
 
             if not tasks:
-                got_empty_list += 1
+                continue
 
-            if got_empty_list == max_loops:
-                break
+            njobs_inqueue = tasks[0].manager.qadapter.get_njobs_in_queue()
+            if njobs_inqueue is None:
+                print('Cannot get njobs_inqueue, going back to sleep')
+                continue
 
-            for task in tasks:
+            #if len(tasks) > 0:
+            #    n_jobs_in_queue = tasks[0].manager.qadapter.get_njobs_in_queue()
+            #    if n_jobs_in_queue is None:
+            #        n_jobs_in_queue = 0
+            #    n_to_run = self.max_jobs - n_jobs_in_queue
+            #else:
+            #    n_to_run = 0
+
+            rest = self.max_njobs_inqueue - njobs_inqueue
+            if rest <= 0:
+                print('too many jobs in the queue, going back to sleep')
+                continue
+
+            stop = len(tasks) if rest > len(tasks) else rest
+            #print("Will fire %d jobs" % stop)
+
+            for task in tasks[:stop]:
                 fired = task.start()
-
                 if fired:
                     launched.append(task)
                     num_launched += 1
 
                 if num_launched == max_nlaunch:
-                    # Exit the outermst loop.
-                    num_loops = max_loops
+                    # Exit the outermost loop.
+                    print('num_launched == max_nlaunch, going back to sleep')
                     break
 
-            #time.sleep(sleep_time)
-            num_loops += 1
 
         # Update the database.
-        self.flow.check_status()
         self.flow.pickle_dump()
 
         return num_launched
@@ -352,6 +370,10 @@ def boxed(msg, ch="=", pad=5):
                       "", ])
 
 
+class PyFlowSchedulerError(Exception):
+    """Exceptions raised by `PyFlowScheduler`."""
+
+
 class PyFlowScheduler(object):
     """
     This object schedules the submission of the tasks in an `AbinitFlow`.
@@ -379,6 +401,10 @@ class PyFlowScheduler(object):
     # Configuration file.
     YAML_FILE = "scheduler.yml"
 
+    DEBUG = 0
+
+    Error = PyFlowSchedulerError
+
     def __init__(self, **kwargs):
         """
         Args:
@@ -392,8 +418,14 @@ class PyFlowScheduler(object):
                 number of minutes to wait
             seconds:
                 number of seconds to wait
+
             verbose:
                 (int) verbosity level
+            max_njobs_inque:
+                Limit on the number of jobs that can be present in the queue
+            use_dynamic_manager:
+                True if the task manager must be re-initialized from 
+                file before launching the jobs. Default: False
         """
         # Options passed to the scheduler.
         self.sched_options = AttrDict(
@@ -406,23 +438,28 @@ class PyFlowScheduler(object):
         )
 
         if all(not v for v in self.sched_options.values()):
-            raise ValueError("Wrong set of options passed to the scheduler.")
+            raise self.Error("Wrong set of options passed to the scheduler.")
 
         self.mailto = kwargs.pop("mailto", None)
         self.verbose = int(kwargs.pop("verbose", 0))
+        self.use_dynamic_manager = kwargs.pop("use_dynamic_manager", False)
+        self.max_njobs_inqueue = kwargs.pop("max_njobs_inqueue", -1)
 
-        self.REMINDME_S = float(kwargs.pop("REMINDME_S", 24 * 3600))
+        self.REMINDME_S = float(kwargs.pop("REMINDME_S", 4 * 24 * 3600))
         self.MAX_NUM_PYEXCS = int(kwargs.pop("MAX_NUM_PYEXCS", 0))
         self.MAX_NUM_ABIERRS = int(kwargs.pop("MAX_NUM_ABIERRS", 0))
-        self.SAFETY_RATIO = int(kwargs.pop("SAFETY_RATIO", 3))
+        self.SAFETY_RATIO = int(kwargs.pop("SAFETY_RATIO", 5))
         #self.MAX_ETIME_S = kwargs.pop("MAX_ETIME_S", )
 
         if kwargs:
-            raise ValueError("Unknown arguments %s" % kwargs)
+            raise self.Error("Unknown arguments %s" % kwargs)
 
-        from apscheduler.scheduler import Scheduler
-
-        self.sched = Scheduler(standalone=True)
+        if has_sched_v3:
+            from apscheduler.schedulers.blocking import BlockingScheduler
+            self.sched = BlockingScheduler()
+        else:
+            from apscheduler.scheduler import Scheduler
+            self.sched = Scheduler(standalone=True)
 
         self.nlaunch = 0
         self.num_reminders = 1
@@ -437,8 +474,15 @@ class PyFlowScheduler(object):
     def from_file(cls, filepath):
         """Read the configuration parameters from a Yaml file."""
         with open(filepath, "r") as fh:
-            d = myaml.load(fh)
-            return cls(**d)
+            return cls(**yaml.load(fh))
+
+    @classmethod
+    def from_string(cls, s):
+        """Create an istance from string s containing a YAML dictionary."""
+        stream = StringIO.StringIO(s)
+        stream.seek(0)
+
+        return cls(**yaml.load(stream))
 
     @classmethod
     def from_user_config(cls):
@@ -465,7 +509,7 @@ class PyFlowScheduler(object):
             return cls.from_file(path)
 
         err_msg = "Cannot locate %s neither in current directory nor in %s" % (cls.YAML_FILE, dirpath)
-        raise RuntimeError(err_msg)
+        raise cls.Error(err_msg)
 
     def __str__(self):
         """String representation."""
@@ -511,9 +555,9 @@ class PyFlowScheduler(object):
         return timedelta(seconds=(time.time() - self.start_time))
 
     def add_flow(self, flow):
-        """Add an `AbinitFlow` to the scheduler."""
+        """Add an `AbinitFlow` flow to the scheduler."""
         if hasattr(self, "_flow"):
-            raise ValueError("Only one flow can be added to the scheduler.")
+            raise self.Error("Only one flow can be added to the scheduler.")
 
         pid_file = os.path.join(flow.workdir, "_PyFlowScheduler.pid")
 
@@ -527,13 +571,13 @@ class PyFlowScheduler(object):
                 "   2) The previous scheduler didn't exit in a clean way.\n\n"
                 "To solve case 1:\n"
                 "   Kill the previous scheduler (use 'kill pid' where pid is the number reported in the file)\n"
-                "   Then you run can start the new scheduler.\n\n"
+                "   Then you can restart the new scheduler.\n\n"
                 "To solve case 2:\n"
                 "   Remove the pid_file and restart the scheduler.\n\n"
                 "Exiting\n" % pid_file
             )
 
-            raise RuntimeError(err_msg)
+            raise self.Error(err_msg)
 
         with open(pid_file, "w") as fh:
             fh.write(str(self.pid))
@@ -549,33 +593,51 @@ class PyFlowScheduler(object):
         self.history.append("Started on %s" % time.asctime())
         self.start_time = time.time()
 
-        self.sched.add_interval_job(self.callback, **self.sched_options)
+        if has_sched_v3:
+            self.sched.add_job(self.callback, 'interval', **self.sched_options)
+        else:
+            self.sched.add_interval_job(self.callback, **self.sched_options)
+
 
         # Try to run the job immediately. If something goes wrong
         # return without initializing the scheduler.
-        retcode = self._runem_all()
+        self._runem_all()
 
-        if retcode:
+        if self.exceptions:
             self.cleanup()
-            self.send_email(msg="Error while trying to run the flow for the first time!")
+            self.send_email(msg="Error while trying to run the flow for the first time!\n %s" % self.exceptions)
 
         self.sched.start()
 
     def _runem_all(self):
         """
-        This function tries to run all the tasks that can be submitted.
+        This function checks the status of all tasks,
+        tries to fix tasks that went unconverged, abicritical, or queuecritical
+        and tries to run all the tasks that can be submitted.+
         """
-        max_nlaunch, excs = -1, []
-
+        max_nlaunch, excs = 10, []
         flow = self.flow
+
+        #nqjobs = flow.get_njobs_inqueue()
+        #if njobs >= self.max_njobs_inqueue:
+        #   return
+
+        # Allow to change the manager at run-time
+        if self.use_dynamic_manager:
+            from pymatgen.io.abinitio.tasks import TaskManager
+            new_manager = TaskManager.from_user_config()
+            for work in flow:
+                work.set_manager(new_manager)
+
+        # check stattus
         flow.check_status()
 
+        # fix problems
         # Try to restart the unconverged tasks
+        # todo donot fire here but prepare for fireing in rapidfire
         for task in self.flow.unconverged_tasks:
             try:
-                msg = "AbinitFlow will try restart task %s" % task
-                print(msg)
-                logger.info(msg)
+                logger.info("AbinitFlow will try restart task %s" % task)
 
                 fired = task.restart()
                 if fired: self.nlaunch += 1
@@ -583,6 +645,11 @@ class PyFlowScheduler(object):
             except Exception:
                 excs.append(straceback())
 
+        # move here from withing rapid fire ...
+        # fix only prepares for restarting, and sets to ready
+        flow.fix_critical()
+
+        # update database
         flow.pickle_dump()
 
         #if self.num_restarts == self.max_num_restarts:
@@ -593,7 +660,7 @@ class PyFlowScheduler(object):
 
         # Submit the tasks that are ready.
         try:
-            nlaunch = PyLauncher(flow).rapidfire(max_nlaunch=max_nlaunch)
+            nlaunch = PyLauncher(flow).rapidfire(max_nlaunch=max_nlaunch, sleep_time=10)
             self.nlaunch += nlaunch
 
             if nlaunch:
@@ -602,8 +669,7 @@ class PyFlowScheduler(object):
         except Exception:
             excs.append(straceback())
 
-        show_status = (self.verbose or flow.num_errored_tasks or flow.num_unconverged_tasks)
-        if show_status: flow.show_status()
+        flow.show_status()
 
         if excs:
             logger.critical("*** Scheduler exceptions:\n *** %s" % "\n".join(excs))
@@ -620,11 +686,17 @@ class PyFlowScheduler(object):
 
     def _callback(self):
         """The actual callback."""
+        if self.DEBUG:
+            # Show the number of open file descriptors
+            print(">>>>> _callback: Number of open file descriptors: %s" % get_open_fds())
+
         self._runem_all()
 
         # Mission accomplished. Shutdown the scheduler.
         all_ok = self.flow.all_ok
-        if self.verbose: print("all_ok", all_ok)
+        if self.verbose:
+            print("all_ok", all_ok)
+
         if all_ok:
             self.shutdown(msg="All tasks have reached S_OK. Will shutdown the scheduler and exit")
 
@@ -637,12 +709,13 @@ class PyFlowScheduler(object):
         if delta_etime.total_seconds() > self.num_reminders * self.REMINDME_S:
             self.num_reminders += 1
             msg = ("Just to remind you that the scheduler with pid %s, flow %s\n has been running for %s " %
-                   (self.pid, self.flow, delta_etime))
+                  (self.pid, self.flow, delta_etime))
             retcode = self.send_email(msg, tag="[REMINDER]")
 
             if retcode:
                 # Cannot send mail, shutdown now!
-                msg += "\nThe scheduler tried to send an e-mail to remind user but send_email returned %d. Aborting now" % retcode
+                msg += ("\nThe scheduler tried to send an e-mail to remind the user\n" +
+                        " but send_email returned %d. Aborting now" % retcode)
                 err_msg += msg
 
         #if delta_etime.total_seconds() > self.MAX_ETIME_S:
@@ -651,14 +724,15 @@ class PyFlowScheduler(object):
         # Too many exceptions. Shutdown the scheduler.
         if self.num_excs > self.MAX_NUM_PYEXCS:
             msg = "Number of exceptions %s > %s. Will shutdown the scheduler and exit" % (
-            self.num_excs, self.MAX_NUM_PYEXCS)
+                self.num_excs, self.MAX_NUM_PYEXCS)
             err_msg += boxed(msg)
 
         # Paranoid check: disable the scheduler if we have submitted
-        # too many jobs (it might be due to some bug or other external reasons such as race conditions betwee difference callbacks.!)
+        # too many jobs (it might be due to some bug or other external reasons 
+        # such as race conditions between difference callbacks!)
         if self.nlaunch > self.SAFETY_RATIO * self.flow.num_tasks:
             msg = "Too many jobs launched %d. Total number of tasks = %s, Will shutdown the scheduler and exit" % (
-            self.nlaunch, self.flow.num_tasks)
+                self.nlaunch, self.flow.num_tasks)
             err_msg += boxed(msg)
 
         # Count the number of tasks with status == S_ERROR.
@@ -708,8 +782,12 @@ class PyFlowScheduler(object):
             self.history.append("Completed on %s" % time.asctime())
             self.history.append("Elapsed time %s" % self.get_delta_etime())
 
+            if self.DEBUG:
+                print(">>>>> shutdown: Number of open file descriptors: %s" % get_open_fds())
+
             retcode = self.send_email(msg)
-            print("send_mail retcode", retcode)
+            if self.DEBUG:
+                print("send_mail retcode", retcode)
 
         finally:
             # Write file with the list of exceptions:
@@ -717,6 +795,7 @@ class PyFlowScheduler(object):
                 dump_file = os.path.join(self.flow.workdir, "_exceptions")
                 with open(dump_file, "w") as fh:
                     fh.writelines(self.exceptions)
+                    fh.write("Shutdown message:\n%s" % msg)
 
             # Shutdown the scheduler thus allowing the process to exit.
             self.sched.shutdown(wait=False)
@@ -755,18 +834,15 @@ class PyFlowScheduler(object):
             # Report the list of exceptions.
             strio.writelines(self.exceptions)
 
-        text = strio.getvalue()
-        #print("text", text)
-
         if tag is None:
             tag = " [ALL OK]" if self.flow.all_ok else " [WARNING]"
 
-        return sendmail(subject=self.flow.name + tag, text=text, mailto=self.mailto)
+        return sendmail(subject=self.flow.name + tag, text=strio.getvalue(), mailto=self.mailto)
 
 
 def sendmail(subject, text, mailto, sender=None):
     """
-    Sends an e-mail either with unix sendmail.
+    Sends an e-mail with unix sendmail.
 
     Args:
         subject:
@@ -811,9 +887,47 @@ def sendmail(subject, text, mailto, sender=None):
     return len(errdata)
 
 
+@deprecated("qadapter.get_njobs_in_queue")
+def get_running_jobs():
+    """
+    Return the number of running jobs.
+
+    ..warning: only slurm is supported
+    """
+    try:
+        import os
+        import subprocess
+        from subprocess import PIPE
+        name = os.environ['LOGNAME']
+        cmd = ['qstat', '-u' + name]
+        data = subprocess.Popen(cmd, stdout=PIPE).communicate()[0]
+        n = len(data.splitlines()) - 1
+    except OSError:
+        n = 0
+
+    return n
+
 # Test for sendmail
-#def sendmail_test():
+#def test_sendmail():
 #    text = "hello\nworld"""
 #    mailto = "matteo.giantomassi@uclouvain.be"
 #    retcode = sendmail("sendmail_test", text, mailto)
 #    print("Retcode", retcode)
+
+
+def get_open_fds():
+    """
+    return the number of open file descriptors for current process
+
+    .. warning: will only work on UNIX-like os-es.
+    """
+    import subprocess
+    import os
+
+    pid = os.getpid()
+    procs = subprocess.check_output(["lsof", '-w', '-Ff', "-p", str(pid)])
+
+    nprocs = len(filter(lambda s: s and s[0] == 'f' and s[1:].isdigit(), procs.split('\n')))
+
+    return nprocs
+
