@@ -6,6 +6,7 @@ import os.path
 from pymatgen.core.units import ArrayWithUnit
 from pymatgen.core.structure import Structure
 from monty.dev import requires
+import collections
 
 
 __author__ = "Matteo Giantomassi"
@@ -142,6 +143,9 @@ class NetcdfReader(object):
             cmode:
                 if cmode=="c", a complex ndarrays is constructed and returned
                 (netcdf does not provide native support from complex datatype).
+
+        Returns:
+            numpy array if varname represents an array, scalar otherwise.
         """
         try:
             var = self.read_variable(varname, path=path)
@@ -150,7 +154,12 @@ class NetcdfReader(object):
 
         if cmode is None:
             # scalar or array
-            return var[0] if not var.shape else var[:]
+            # getValue is not portable!
+            try:
+                return var.getValue()[0] if not var.shape else var[:]
+            except IndexError:
+                return var.getValue() if not var.shape else var[:]
+
         else:
             assert var.shape[-1] == 2
             if cmode == "c":
@@ -279,8 +288,7 @@ def structure_from_etsf_file(ncdata, site_properties=None):
     ncdata, closeit = as_ncreader(ncdata)
 
     # TODO check whether atomic units are used
-    lattice = ArrayWithUnit(ncdata.read_value("primitive_vectors"),
-                            "bohr").to("ang")
+    lattice = ArrayWithUnit(ncdata.read_value("primitive_vectors"), "bohr").to("ang")
 
     red_coords = ncdata.read_value("reduced_atom_positions")
     natom = len(red_coords)
