@@ -6,7 +6,7 @@ import collections
 import shutil
 import operator
 
-from pymatgen.util.string_utils import list_strings, StringColorizer, WildCard
+from pymatgen.util.string_utils import list_strings, WildCard
 
 import logging
 from six.moves import filter
@@ -160,9 +160,9 @@ class Directory(object):
         """Recursively delete the directory tree"""
         shutil.rmtree(self.path, ignore_errors=True)
 
-    def path_in(self, filename):
+    def path_in(self, file_basename):
         """Return the absolute path of filename in the directory."""
-        return os.path.join(self.path, filename)
+        return os.path.join(self.path, file_basename)
 
     def list_filepaths(self, wildcard=None):
         """
@@ -237,6 +237,7 @@ _EXT2VARS = {
     "DKK": {},
 }
 
+
 def irdvars_for_ext(ext):
     """
     Returns a dictionary with the ABINIT variables 
@@ -284,7 +285,7 @@ def abi_splitext(filename):
 
     root = filename[:i]
     if is_ncfile: 
-        ext = ext + ".nc"
+        ext += ".nc"
 
     return root, ext
 
@@ -375,14 +376,17 @@ def _bop_not(obj):
     """Boolean not."""
     return not bool(obj)
 
+
 def _bop_and(obj1, obj2):
     """Boolean and."""
     return bool(obj1) and bool(obj2)
 
+
 def _bop_or(obj1, obj2):
     """Boolean or."""
     return bool(obj1) or bool(obj2)
 
+
 def _bop_divisible(num1, num2):
     """Return True if num1 is divisible by num2."""
     return (num1 % num2) == 0.0
@@ -511,7 +515,7 @@ def evaluate_rpn(rpn):
 
 class Condition(object):
     """
-    This object receive a dictionary that defines a boolean condition whose syntax is similar
+    This object receives a dictionary that defines a boolean condition whose syntax is similar
     to the one used in mongodb (albeit not all the operators available in mongodb are supported here).
 
     Example:
@@ -543,8 +547,8 @@ class Condition(object):
     def apply(self, obj):
         try:
             return evaluate_rpn(map2rpn(self.cmap, obj))
-        except:
-            logger.warning("Condition.apply() raise Exception")
+        except Exception as exc:
+            logger.warning("Condition.apply() raise Exception:\n %s" % str(exc))
             return False
 
 
