@@ -13,8 +13,10 @@ __date__ = "Jul 24, 2012"
 
 import numpy as np
 from fractions import gcd, Fraction
-
 from itertools import groupby
+
+import six
+
 from pymatgen.core.structure import Specie, Composition
 from pymatgen.core.periodic_table import get_el_sp
 from pymatgen.transformations.transformation_abc import AbstractTransformation
@@ -29,7 +31,7 @@ from pymatgen.structure_prediction.substitution_probability import \
 from pymatgen.analysis.structure_matcher import StructureMatcher, \
     SpinComparator
 from pymatgen.analysis.energy_models import SymmetryModel
-from pymatgen.serializers.json_coders import PMGJSONDecoder
+from monty.json import MontyDecoder
 
 
 class ChargeBalanceTransformation(AbstractTransformation):
@@ -73,8 +75,7 @@ class ChargeBalanceTransformation(AbstractTransformation):
     def is_one_to_many(self):
         return False
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         return {"name": self.__class__.__name__, "version": __version__,
                 "init_args": {"charge_balance_sp": self._charge_balance_sp},
                 "@module": self.__class__.__module__,
@@ -134,8 +135,7 @@ class SuperTransformation(AbstractTransformation):
     def is_one_to_many(self):
         return True
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         return {"name": self.__class__.__name__, "version": __version__,
                 "init_args": {
                     "transformations": self._transformations,
@@ -235,8 +235,7 @@ class MultipleSubstitutionTransformation(object):
     def is_one_to_many(self):
         return True
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         return {"name": self.__class__.__name__, "version": __version__,
                 "init_args": {
                     "sp_to_replace": self._sp_to_replace,
@@ -375,8 +374,7 @@ class EnumerateStructureTransformation(AbstractTransformation):
     def is_one_to_many(self):
         return True
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         return {"name": self.__class__.__name__, "version": __version__,
                 "init_args": {"symm_prec": self.symm_prec,
                               "min_cell_size": self.min_cell_size,
@@ -437,8 +435,7 @@ class SubstitutionPredictorTransformation(AbstractTransformation):
     def is_one_to_many(self):
         return True
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         d = {"name": self.__class__.__name__, "version": __version__,
              "init_args": self._kwargs, "@module": self.__class__.__module__,
              "@class": self.__class__.__name__}
@@ -494,7 +491,7 @@ class MagOrderingTransformation(AbstractTransformation):
         atom_per_specie = [structure.composition.get(m)
                            for m in mag_species_spin.keys()]
 
-        n_gcd = reduce(gcd, atom_per_specie)
+        n_gcd = six.moves.reduce(gcd, atom_per_specie)
 
         if not n_gcd:
             raise ValueError(
@@ -528,7 +525,7 @@ class MagOrderingTransformation(AbstractTransformation):
             MagOrderingTransformation.determine_min_cell(
                 structure, self.mag_species_spin,
                 self.order_parameter)),
-            enum_args.get("min_cell_size"))
+            enum_args.get("min_cell_size", 1))
 
         max_cell = self.enum_kwargs.get('max_cell_size')
         if max_cell:
@@ -579,13 +576,12 @@ class MagOrderingTransformation(AbstractTransformation):
     def is_one_to_many(self):
         return True
 
-    @property
-    def to_dict(self):
+    def as_dict(self):
         return {
             "name": self.__class__.__name__, "version": __version__,
             "init_args": {"mag_species_spin": self.mag_species_spin,
                           "order_parameter": self.order_parameter,
-                          "energy_model": self.emodel.to_dict,
+                          "energy_model": self.emodel.as_dict(),
                           "enum_kwargs": self.enum_kwargs},
             "@module": self.__class__.__module__,
             "@class": self.__class__.__name__}
@@ -595,6 +591,6 @@ class MagOrderingTransformation(AbstractTransformation):
         init = d["init_args"]
         return MagOrderingTransformation(
             init["mag_species_spin"], init["order_parameter"],
-            energy_model=PMGJSONDecoder().process_decoded(
+            energy_model=MontyDecoder().process_decoded(
                 init["energy_model"]),
             **init["enum_kwargs"])
